"""
Gas dynamics calculations.

The following notation is used throughout the different modules:

* `X_0` property X at the stagnation point.
* `X_star` property X at the critical point.
* `X_1` upstream property X (when applicable).
* `X_2` downstream property X (when applicable).
* `X_X0` ratio between `X` and `X_0`.
* `X2_X1` ratio between `X_2` and `X_1`.

"""
