"""
===========
scikit-aero
===========

Aeronautical engineering calculations in Python

"""

# Package version
__version__ = "0.2.dev0"

# Prellocate units module
import pint

units = pint.UnitRegistry()
